# coding: utf-8

"""
    Geometry Backend API v2

    The ShapeDiver Geometry Backend system is used to: * host Grasshopper models in a secure, reliable, scalable, and performant way, * run computations of Grasshopper models, * and cache and output the results of computations and exports.

    The version of the OpenAPI document: 2.13.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from shapediver.geometry_api_v2.client.models.req_model_file_type import ReqModelFileType
from shapediver.geometry_api_v2.client.models.req_trust_level import ReqTrustLevel
from typing import Optional, Set
from typing_extensions import Self

class ReqModel(BaseModel):
    """
    Body of a template request.
    """ # noqa: E501
    accessdomains: Optional[List[StrictStr]] = Field(default=None, description="List of domains (origins) this model is limited to (is ignored in case `pub` is true).")
    auth_groups: Optional[List[StrictStr]] = Field(default=None, description="List of allowed auth-groups.")
    backendaccess: Optional[StrictBool] = Field(default=None, description="Should backend access to the model be allowed.")
    filename: Optional[StrictStr] = Field(default=None, description="Original name of the model's grasshopper file.")
    ftype: Optional[ReqModelFileType] = Field(default=None, description="File type of the model.")
    initial_warmup: Optional[StrictBool] = Field(default=None, description="Allows to control whether the model should be warmed up immediately after loading by running a computation. This increases the likelihood of following computation requests to be faster.")
    id2: Optional[StrictStr] = Field(default=None, description="Optional second model ID.")
    max_comp_time: Optional[Annotated[int, Field(strict=True, ge=0)]] = Field(default=None, description="Maximum number of milliseconds allowed for computations of this model.")
    max_export_size: Optional[Annotated[int, Field(strict=True, ge=0)]] = Field(default=None, description="Maximum number of bytes allowed to be exported from a model for a specific set of parameter values.")
    max_idle_minutes: Optional[Annotated[int, Field(strict=True, ge=0)]] = Field(default=None, description="Maximum amount of minutes a loaded model may be unused before it gets unloaded.  Note: Models may get unloaded earlier than that.")
    max_model_size: Optional[Annotated[int, Field(strict=True, ge=0)]] = Field(default=None, description="Maximum number of bytes allowed for the model's Grasshopper file size.")
    max_output_size: Optional[Annotated[int, Field(strict=True, ge=0)]] = Field(default=None, description="Maximum number of bytes allowed for an output of a model for a specific set of parameter values.")
    webhook_token: Optional[StrictBool] = Field(default=None, description="Allows to control whether the model's Grasshopper file can contain scripts.")
    max_wait_time: Optional[Annotated[int, Field(strict=True, ge=0)]] = Field(default=None, description="Maximum time a computation request may stay waiting before a further worker goes ahead regardless of whether it already has the model loaded, and regardless of `num_loaded_max`. This allows to configure a soft or a hard upper boundary for the number of loaded models.")
    name: Optional[StrictStr] = Field(default=None, description="Name of the model.")
    num_loaded_max: Optional[Annotated[int, Field(strict=True, ge=0)]] = Field(default=None, description="Maximum number of workers that should have the model loaded at the same time.")
    num_loaded_min: Optional[Annotated[int, Field(strict=True, ge=0)]] = Field(default=None, description="Minimum number of workers which should have the model loaded once a session to the model gets opened.")
    num_preloaded_min: Optional[Annotated[int, Field(strict=True, ge=0)]] = Field(default=None, description="Minimum number of workers that should always have the model loaded, regardless of session activity. This allows to minimise the likelihood of computation requests being slower due to model loading.")
    org_id: Optional[StrictStr] = Field(default=None, description="Optional organization ID.")
    prev_id: Optional[StrictStr] = Field(default=None, description="Model ID of this model's previous version.")
    pub: StrictBool = Field(description="Allows public access (ignore `accessdomains`).")
    require_iframe: Optional[StrictBool] = Field(default=None, description="Enforces iframe embedding instead of direct embedding.")
    require_token: Optional[StrictBool] = Field(default=None, description="Enforces token-based authentication for this model.")
    session_rate_limit: Optional[Annotated[int, Field(strict=True, ge=0)]] = Field(default=None, description="Limits the number of sessions that can be created by a specific IP address in one hour.")
    trust: Optional[ReqTrustLevel] = Field(default=None, description="Can this model be trusted (controls whether failed computations will be retried).")
    use_cdn: Optional[StrictBool] = Field(default=None, description="Allows the usage of the CDN for fast content distribution.")
    user_id: Optional[StrictStr] = Field(default=None, description="Optional user ID.")
    webhook_url: Optional[StrictStr] = Field(default=None, description="The webhook-url for updating the platform backend about model status changes.")
    __properties: ClassVar[List[str]] = ["accessdomains", "auth_groups", "backendaccess", "filename", "ftype", "initial_warmup", "id2", "max_comp_time", "max_export_size", "max_idle_minutes", "max_model_size", "max_output_size", "webhook_token", "max_wait_time", "name", "num_loaded_max", "num_loaded_min", "num_preloaded_min", "org_id", "prev_id", "pub", "require_iframe", "require_token", "session_rate_limit", "trust", "use_cdn", "user_id", "webhook_url"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ReqModel from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ReqModel from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "accessdomains": obj.get("accessdomains"),
            "auth_groups": obj.get("auth_groups"),
            "backendaccess": obj.get("backendaccess"),
            "filename": obj.get("filename"),
            "ftype": obj.get("ftype"),
            "initial_warmup": obj.get("initial_warmup"),
            "id2": obj.get("id2"),
            "max_comp_time": obj.get("max_comp_time"),
            "max_export_size": obj.get("max_export_size"),
            "max_idle_minutes": obj.get("max_idle_minutes"),
            "max_model_size": obj.get("max_model_size"),
            "max_output_size": obj.get("max_output_size"),
            "webhook_token": obj.get("webhook_token"),
            "max_wait_time": obj.get("max_wait_time"),
            "name": obj.get("name"),
            "num_loaded_max": obj.get("num_loaded_max"),
            "num_loaded_min": obj.get("num_loaded_min"),
            "num_preloaded_min": obj.get("num_preloaded_min"),
            "org_id": obj.get("org_id"),
            "prev_id": obj.get("prev_id"),
            "pub": obj.get("pub"),
            "require_iframe": obj.get("require_iframe"),
            "require_token": obj.get("require_token"),
            "session_rate_limit": obj.get("session_rate_limit"),
            "trust": obj.get("trust"),
            "use_cdn": obj.get("use_cdn"),
            "user_id": obj.get("user_id"),
            "webhook_url": obj.get("webhook_url")
        })
        return _obj


